@@ -11,8 +11,8 @@ package Spreadsheet::Edit::IO;
 
 # Allow "use <thismodule. VERSION ..." in development sandbox to not bomb
 { no strict 'refs'; ${__PACKAGE__."::VER"."SION"} = 1999.999; }
-our $VERSION = '1000.018'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
-our $DATE = '2024-10-08'; # DATE from Dist::Zilla::Plugin::OurDate
+our $VERSION = '1000.019'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
+our $DATE = '2024-10-14'; # DATE from Dist::Zilla::Plugin::OurDate
 
 # This module is derived from the old never-released Text:CSV::Spreadsheet
 
@@ -56,6 +56,7 @@ use List::Util qw/none all notall first min max/;
 use Encode qw(encode decode);
 use File::Glob qw/bsd_glob GLOB_NOCASE/;
 use Digest::MD5 qw/md5_base64/;
+use Data::Hexify qw/Hexify/;
 use Text::CSV ();
 # DDI 5.025 is needed for Windows-aware qsh()
 use Data::Dumper::Interp 5.025 qw/vis visq dvis dvisq ivis ivisq avis qsh qshlist u/;
@@ -122,7 +123,7 @@ sub _get_exclusive_lock($) { # returns lock object
   my $sleeptime = 1;
   my $lock_fh;
   while (! defined $lock_fh) {
-    #warn "$$ : ### AAA open $lockfile_path ...\n";
+    #warn "$$ : ### AA1 open $lockfile_path ...\n";
     open $lock_fh, "+>>", $lockfile_path or die $!;
     #warn "$$ : ### AA2 open succeeded.\n";
     eval { chmod 0666, $lock_fh; }; # sometimes not implemented
@@ -794,7 +795,7 @@ sub _convert_using_openlibre($$$) {
       #  effectively disabling Token 8.  This must be false to recognize dates
       #   like "Jan 1, 2000" which by necessity must be quoted for the comma,
       #   but will **CORRUPT** zip codes with leading zeroes unless
-      #   col_formats overrides (which it does now by default).
+      #   {col_formats} overrides (which it does now by default).
       .",false" # default: false
       # Token 8: on input: "Detect Special Numbers", i.e. date or time values
       #   in human form, numbers in scientific (expondntial) notation etc.
@@ -821,6 +822,7 @@ sub _convert_using_openlibre($$$) {
     if ($opts->{cvt_to} eq "csv") {
       my $filter_name = $suf2ofilter->{$opts->{cvt_to}} or oops;
       my $enc = $opts->{output_encoding};
+oops dvis '$opts' unless defined $enc;
       my $charset = _name2LOcharsetnum($enc); # dies if unknown enc
       $filter_name.":"
       # Tokens 1-4: FldSep=, TxtDelim=" Charset FirstLineNum
@@ -1223,14 +1225,37 @@ sub _preprocess($$) {
   my ($fh, $start_pos);
   # Skip to ==BODY== below
 
+#  my sub _dump_fh($) {
+#    return unless $debug;
+#    my $tag = shift;
+#    confess dvis "($tag) fh is not open \$start_pos" unless $fh;
+#    my @layers = PerlIO::get_layers($fh);
+#    my $cur_pos = tell($fh);
+#    if (! seek($fh, $start_pos//0, SEEK_SET)) {
+#      btwN 1,dvis "($tag) **NOT SEEKABLE** \$cur_pos \@layers \$start_pos";
+#    } else {
+#      if ((my $n = read $fh, my $buffer, 128) != 0) {
+#        btwN 1,dvis "($tag) \$cur_pos \$start_pos \@layers First $n items are:\n", Hexify($buffer), "\n";
+#      } else {
+#        btwN 1,dvis "($tag) \$cur_pos \$start_pos \@layers **APPEARS EMPTY**";
+#      }
+#      seek($fh, $cur_pos, SEEK_SET) or die "re-seek to $cur_pos: $!";
+#    }
+#  }
   my sub set_fh_encoding() { # returns true if set
     my $enc = $opts->{input_encoding}; # must already be resolved!
     my $bmode = ":raw:encoding($enc):crlf";
     binmode $fh, $bmode or die "binmode '$bmode' : $!";
+    if ($debug) {
+      my @layers = PerlIO::get_layers($fh);
+      warn dvis 'set_fh_encoding: $fh @layers\n';
+    }
   }
   my sub open_input() {
+    oops if defined $fh;
     if (defined $$ref2octets) {
       open $fh, "<:raw", $ref2octets or confess "BUG:in-mem open:$!";
+      #_dump_fh("open_input TO PREVIOUSLY SLURPED");
     } else {
       my $path = $opts->{inpath_sans_sheet};
       $fh = openhandle($path); # undef unless $path is a file handle
@@ -1238,19 +1263,26 @@ sub _preprocess($$) {
         open $fh, "<", $path or die "$path : $!";
       }
       binmode($fh);
+      #_dump_fh("AAA $path open_input raw");
     }
     if (! seek($fh, 0, SEEK_SET)) {
       oops if defined $$ref2octets;
       local $/ = undef;
       $$ref2octets = <$fh>;  # N.B. includes possible BOM
+      close $fh;
+      $fh = undef;
       open $fh, "<:raw", $ref2octets or confess "BUG:in-mem open:$!";
+      #_dump_fh("BBB unseekable, slurped");
     }
     my $bomenc = File::BOM::get_encoding_from_filehandle($fh);
     $start_pos = tell($fh);
     if ($bomenc) {
       btw "Input has BOM, bomenc=$bomenc\n" if $debug;
       $opts->{input_encoding} = $bomenc;
+      binmode($fh);
+      binmode($fh, ":raw:encoding($bomenc):crlf") or die "binmode: $!";
     }
+    #_dump_fh("CCC final");
   }
   my sub determine_input_encoding() {
     # If one encoding was specified by the user or implied by a BOM, use it;
@@ -1313,7 +1345,7 @@ sub _preprocess($$) {
       SEP:
       for my $sep (defined($opts->{sep_char})
                      ? ($opts->{sep_char}) : (",","\t")) {
-        btw dvis '--- TRYING $q $sep ---' if $debug;
+        btw dvisq '--- TRYING $q $sep ---' if $debug;
 
 #        # Preliminary check for an illegal use of the quote char
 #        if (defined($chars)
@@ -1331,12 +1363,31 @@ sub _preprocess($$) {
           $opts->{sep_char} = $sep;
           last Q;
         }
-        warn vis '$@\nq=$q sep=$sep did not work...\n' if $debug;
+        warn dvis('$q sep=$sep did not work...\n'),vis($@),"\n"
+          if $debug;
+        _dump_fd("$q sep=$sep did not work") if $debug;
+        $$r2rows = undef;
       }
     }
     unless (defined($$r2rows)) {
+      #confess "Input file is not valid CSV (or we have a bug)\n"
+      seek($fh, $start_pos, SEEK_SET) or die $!; # skip over possible BOM
+      my $somechars = substr(do {local $/; <$fh>}, 0, 100);
+      my @layers = PerlIO::get_layers($fh);
+      warn dvis '$start_pos @layers $somechars\n';
+      if (open my $fh99, "<:raw", $opts->{inpath}) {
+        if ((my $nb = read $fh99, my $octets, 128) != 0) {
+          warn "First $nb octets are as follows:\n", Hexify($octets), "\n";
+        } else {
+          warn "File appears to be EMPTY\n";
+        }
+      }
       confess "Input file is not valid CSV (or we have a bug)\n"
     }
+    else {
+      warn dvis '#### CSV DETECTED $opts->{quote_char} $opts->{sep_char}'
+        if $debug;
+    }
   }#determine_csv_q_sep
 
   my sub determine_csv_col_formats($) {
@@ -1366,30 +1417,30 @@ sub _preprocess($$) {
         for ($row->[$cx]) {
           # recognize obvious Y/M/D or M/D/Y or D/M/Y date forms
           if (m#\b(?<y>(?:[12]\d)?\d\d)/(?<m>\d\d)/(?<d>\d\d)\b#) {
-            if ($+{d} > 12 && $+{d} <= 31 && $+{m} >= 1 && $+{m} <= 12
-                 && ($+{y} < 100 || $+{y} >= 1000)) {
+            if ($+{d} > 12 && $+{d} <= 31 && $+{"m"} >= 1 && $+{"m"} <= 12
+                 && ($+{"y"} < 100 || $+{"y"} >= 1000)) {
               recognized($cx,$rx,$_,"YY/MM/DD");
               next CX;
             }
             # If ambiguous YYYY/??/?? we can still assume it is a date and not text
-            if (length( $+{y} )==4) {
+            if (length( $+{"y"} )==4) {
               #recognized($cx,$rx,$_,""," as some kind of date, fmt unknown");
               next RX;
             }
           }
           if (m#\b(?<m>\d\d)/(?<d>\d\d)/(?<y>(?:[12]\d)?\d\d)\b#) {
-            if ($+{y} < 100 || $+{y} >= 1000) {
-              if ($+{d} > 12 && $+{d} <= 31 && $+{m} >= 1 && $+{m} <= 12) {
+            if ($+{"y"} < 100 || $+{"y"} >= 1000) {
+              if ($+{d} > 12 && $+{d} <= 31 && $+{"m"} >= 1 && $+{"m"} <= 12) {
                 recognized($cx,$rx,$_,"MM/DD/YY");
                 next CX
               }
-              elsif ($+{m} > 12 && $+{m} <= 31 && $+{d} >= 1 && $+{d} <= 12) {
+              elsif ($+{"m"} > 12 && $+{"m"} <= 31 && $+{d} >= 1 && $+{d} <= 12) {
                 recognized($cx,$rx,$_,"DD/MM/YY");
                 next CX
               }
             }
             # If ambiguous ??/??/YYYY we can still assume it is a date and not text
-            if (length($+{y})==4) {
+            if (length($+{"y"})==4) {
               #recognized($cx,$rx,$_,""," as some kind of date, fmt unknown");
               next RX;
             }
@@ -1447,11 +1498,23 @@ sub _preprocess($$) {
       $opts->{cvt_from} = "csv";
     } else {
       #croak "Can not detect what kind of file ",qsh($opts->{inpath})," is\n";
-      croak "Can not detect what kind of file ",qsh($opts->{inpath})," is\n";
+      carp "Can not detect what kind of file ",qsh($opts->{inpath})," is\n",
+           $@, "\n";
+      open my $fh99, "<:raw", $opts->{inpath} or die "Can not re-open $opts->{inpath}: $!";
+      if ((my $nb = read $fh99, my $octets, 16) != 0) {
+        die "First 16 octets are as follows:\n", Hexify($octets);
+      } else {
+        die "File appears to be EMPTY";
+      }
     }
   }
   oops if defined($rows) && $opts->{cvt_from} ne "csv";
 
+  if ($opts->{cvt_from} eq "csv") {
+    determine_csv_col_formats(\$rows);
+  } else {
+    oops if defined($rows);
+  }
   $opts->{output_encoding} //= $default_output_encoding
     if $opts->{cvt_to} eq "csv";
 
@@ -1587,7 +1650,7 @@ sub _extract_one_csv($) {
       croak "$opts->{inpath_sans_sheet} contains multiple sheets; you must specify a sheetname\n"
     }
   }
-}
+}#_extract_one_csv
 sub _write_spreadsheet($) {
   my ($opts) = @_;
 
